{   Target driver routines that are specific to the PIC16 family.
}
module picprg_16;
define picprg_send6;
define picprg_send14ss;
define picprg_recv14ss;
define picprg_erase_16f;
%include '/cognivision_links/dsee_libs/pics/picprg2.ins.pas';
{
*******************************************************************************
*
*   Subroutine PICPRG_SEND6 (PR, DAT, STAT)
*
*   Send the 6 low bits of DAT to the target.  This is the format of 16F
*   family commands.
}
procedure picprg_send6 (               {send 6 bits of data to the target}
  in out  pr: picprg_t;                {state for this use of the library}
  in      dat: sys_int_conv16_t;       {the data word to send}
  out     stat: sys_err_t);            {completion status}
  val_param;

begin
  picprg_cmdw_send (pr, 6, dat, stat); {send the bits}
  end;
{
*******************************************************************************
*
*   Subroutine PICPRG_SEND14SS (PR, DAT, STAT)
*
*   Send the 14 bit data word in DAT as 16 bits with a leading start and trailing
*   stop bit.
}
procedure picprg_send14ss (            {send 14 data bits with start/stop bits}
  in out  pr: picprg_t;                {state for this use of the library}
  in      dat: sys_int_conv16_t;       {the data word to send}
  out     stat: sys_err_t);            {completion status}
  val_param;

var
  i: sys_int_conv16_t;                 {data word with start and stop bits}

begin
  i := lshft(dat & 16#3FFF, 1);        {make data word with 0 start and stop bits}
  picprg_cmdw_send (pr, 16, i, stat);  {send it}
  end;
{
*******************************************************************************
*
*   Subroutine PICPRG_RECV14SS (PR, DAT, STAT)
*
*   Read a 16 bit data word that includes the start and stop bits, and return
*   the 14 bit data value in DAT.
}
procedure picprg_recv14ss (            {receive 14 data bits with start/stop bits}
  in out  pr: picprg_t;                {state for this use of the library}
  out     dat: sys_int_conv16_t;       {the 14 data bits}
  out     stat: sys_err_t);            {completion status}
  val_param;

var
  i32: sys_int_conv32_t;

begin
  picprg_cmdw_recv (pr, 16, i32, stat);
  if sys_error(stat) then return;
  dat := rshft(i32, 1) & 16#3FFF;      {remove start/stop bits and align 14 bit data}
  end;
{
*******************************************************************************
*
*   Subroutine PICPRG_ERASE_16F (PR, STAT)
*
*   Erase all erasable non-volatile memory in the target chip.
*
*   This is the generic version for 16Fxxx devices.
}
procedure picprg_erase_16f (           {erase routine for generic 16Fxxx}
  in out  pr: picprg_t;                {state for this use of the library}
  out     stat: sys_err_t);            {completion status}
  val_param;

var
  i: sys_int_machine_t;                {scratch integer and loop counter}

begin
  picprg_reset (pr, stat);             {reset target to put it into known state}
  if sys_error(stat) then return;
{
*   Write 0 to the configuration word.  This sets all the code protection
*   bits.  The data memory is not erased unless the code protection bits
*   are actually changed from protected to unprotected, although this is
*   not documented in the Microchip programming spec.  You just gotta
*   know, I guess.
}
  picprg_cmdw_writing (pr, stat);      {indicate the target is being written to}
  if sys_error(stat) then return;

  picprg_send6 (pr, 2#000000, stat);   {send LOAD CONFIGURATION, sets adr to 2000h}
  if sys_error(stat) then return;
  picprg_send14ss (pr, 0, stat);       {send dummy data word}
  if sys_error(stat) then return;

  for i := 1 to 7 do begin             {advance the address to 2007h}
    picprg_send6 (pr, 2#000110, stat); {send INCREMENT ADDRESS command}
    if sys_error(stat) then return;
    end;

  picprg_send6 (pr, 2#000010, stat);   {LOAD DATA FOR PROGRAM MEMORY}
  if sys_error(stat) then return;
  picprg_send14ss (pr, 0, stat);       {send data as all zeros}
  if sys_error(stat) then return;

  picprg_send6 (pr, 2#011000, stat);   {BEGIN PROGRAMMING ONLY CYCLE}
  if sys_error(stat) then return;
  picprg_cmdw_wait (pr, 0.015, stat);  {force delay before next operation}
  if sys_error(stat) then return;
{
*   All the code protection has been enabled.  Now perform the special erase
*   sequence to completely erase a code protected chip.
}
  picprg_reset (pr, stat);             {reset target to put it into known state}
  if sys_error(stat) then return;

  picprg_cmdw_writing (pr, stat);      {indicate the target is being written to}
  if sys_error(stat) then return;

  picprg_send6 (pr, 2#000000, stat);   {send LOAD CONFIGURATION command}
  if sys_error(stat) then return;
  picprg_send14ss (pr, 16#3FFF, stat); {send the data word for this command}
  if sys_error(stat) then return;

  for i := 1 to 7 do begin             {each time to increment the address}
    picprg_send6 (pr, 2#000110, stat); {send INCREMENT ADDRESS command}
    if sys_error(stat) then return;
    end;

  picprg_send6 (pr, 2#000001, stat);   {BULK ERASE SETUP 1}
  if sys_error(stat) then return;
  picprg_send6 (pr, 2#000111, stat);   {BULK ERASE SETUP 2}
  if sys_error(stat) then return;
  picprg_send6 (pr, 2#001000, stat);   {BEGIN ERASE/PROGRAMMING CYCLE}
  if sys_error(stat) then return;

  picprg_cmdw_wait (pr, 0.015, stat);  {force delay before next operation}
  if sys_error(stat) then return;

  picprg_send6 (pr, 2#000001, stat);   {BULK ERASE SETUP 1}
  if sys_error(stat) then return;
  picprg_send6 (pr, 2#000111, stat);   {BULK ERASE SETUP 2}
  if sys_error(stat) then return;
{
*   Done performing special sequence to bulk erase a protected chip.  All
*   eraseable bits are now set to 1.
}
  picprg_reset (pr, stat);             {reset target to guaranteed known state}
  if sys_error(stat) then return;
  end;
